@@ -1,11 +1,53 @@
+"""Find occurrences of a name in a project.
+
+This module consists of a `Finder` that finds all occurrences of a name
+in a project. The `Finder.find_occurrences()` method is a generator that
+yields `Occurrence` instances for each occurrence of the name. To create
+a `Finder` object, use the `create_finder()` function:
+
+    finder = occurrences.create_finder(project, 'foo', pyname)
+    for occurrence in finder.find_occurrences():
+        pass
+
+It's possible to filter the occurrences. They can be specified when
+calling the `create_finder()` function.
+
+  * `only_calls`: If True, return only those instances where the name is
+    a function that's being called.
+
+  * `imports`: If False, don't return instances that are in import
+    statements.
+
+  * `unsure`: If a prediate function, return instances where we don't
+    know what the name references. It also filters based on the
+    predicate function.
+
+  * `docs`: If True, it will search for occurrences in regions normally
+    ignored. E.g., strings and comments.
+
+  * `in_hierarchy`: If True, it will find occurrences if the name is in
+    the class's hierarchy.
+
+  * `instance`: Used only when you want implicit interfaces to be
+    considered.
+
+  * `keywords`: If False, don't return instances that are the names of keyword
+    arguments
+"""
+
+import ast
 import re
 
-import rope.base.pynames
-from rope.base import (pynames, pyobjects, codeanalyze, evaluate,
-                       exceptions, utils, worder)
+from rope.base import codeanalyze
+from rope.base import evaluate
+from rope.base import exceptions
+from rope.base import pynames
+from rope.base import pyobjects
+from rope.base import utils
+from rope.base import worder
 
 
-class Finder(object):
+class Finder:
     """For finding occurrences of a name
 
     The constructor takes a `filters` argument.  It should be a list
@@ -20,8 +62,8 @@ class Finder(object):
 
     """
 
-    def __init__(self, pycore, name, filters=[lambda o: True], docs=False):
-        self.pycore = pycore
+    def __init__(self, project, name, filters=[lambda o: True], docs=False):
+        self.project = project
         self.name = name
         self.docs = docs
         self.filters = filters
@@ -29,8 +71,9 @@ class Finder(object):
 
     def find_occurrences(self, resource=None, pymodule=None):
         """Generate `Occurrence` instances"""
-        tools = _OccurrenceToolsCreator(self.pycore, resource=resource,
-                                        pymodule=pymodule, docs=self.docs)
+        tools = _OccurrenceToolsCreator(
+            self.project, resource=resource, pymodule=pymodule, docs=self.docs
+        )
         for offset in self._textual_finder.find_offsets(tools.source_code):
             occurrence = Occurrence(tools, offset)
             for filter in self.filters:
@@ -42,8 +85,18 @@ class Finder(object):
                 break
 
 
-def create_finder(pycore, name, pyname, only_calls=False, imports=True,
-                  unsure=None, docs=False, instance=None, in_hierarchy=False):
+def create_finder(
+    project,
+    name,
+    pyname,
+    only_calls=False,
+    imports=True,
+    unsure=None,
+    docs=False,
+    instance=None,
+    in_hierarchy=False,
+    keywords=True,
+):
     """A factory for `Finder`
 
     Based on the arguments it creates a list of filters.  `instance`
@@ -51,29 +104,30 @@ def create_finder(pycore, name, pyname, only_calls=False, imports=True,
     considered.
 
     """
-    pynames = set([pyname])
+    pynames_ = {pyname}
     filters = []
     if only_calls:
         filters.append(CallsFilter())
     if not imports:
         filters.append(NoImportsFilter())
-    if isinstance(instance, rope.base.pynames.ParameterName):
+    if not keywords:
+        filters.append(NoKeywordsFilter())
+    if isinstance(instance, pynames.ParameterName):
         for pyobject in instance.get_objects():
             try:
-                pynames.add(pyobject[name])
+                pynames_.add(pyobject[name])
             except exceptions.AttributeNotFoundError:
                 pass
-    for pyname in pynames:
+    for pyname in pynames_:
         filters.append(PyNameFilter(pyname))
         if in_hierarchy:
             filters.append(InHierarchyFilter(pyname))
     if unsure:
         filters.append(UnsureFilter(unsure))
-    return Finder(pycore, name, filters=filters, docs=docs)
+    return Finder(project, name, filters=filters, docs=docs)
 
 
-class Occurrence(object):
-
+class Occurrence:
     def __init__(self, tools, offset):
         self.tools = tools
         self.offset = offset
@@ -97,27 +151,26 @@ class Occurrence(object):
     @utils.saveit
     def get_primary_and_pyname(self):
         try:
-            return self.tools.name_finder.get_primary_and_pyname_at(
-                self.offset)
+            return self.tools.name_finder.get_primary_and_pyname_at(self.offset)
         except exceptions.BadIdentifierError:
             pass
 
     @utils.saveit
     def is_in_import_statement(self):
-        return (self.tools.word_finder.is_from_statement(self.offset) or
-                self.tools.word_finder.is_import_statement(self.offset))
+        return self.tools.word_finder.is_from_statement(
+            self.offset
+        ) or self.tools.word_finder.is_import_statement(self.offset)
 
     def is_called(self):
         return self.tools.word_finder.is_a_function_being_called(self.offset)
 
     def is_defined(self):
-        return self.tools.word_finder.is_a_class_or_function_name_in_header(
-            self.offset)
+        return self.tools.word_finder.is_a_class_or_function_name_in_header(self.offset)
 
     def is_a_fixed_primary(self):
         return self.tools.word_finder.is_a_class_or_function_name_in_header(
-            self.offset) or \
-            self.tools.word_finder.is_a_name_after_from_import(self.offset)
+            self.offset
+        ) or self.tools.word_finder.is_a_name_after_from_import(self.offset)
 
     def is_written(self):
         return self.tools.word_finder.is_assigned_here(self.offset)
@@ -125,6 +178,9 @@ class Occurrence(object):
     def is_unsure(self):
         return unsure_pyname(self.get_pyname())
 
+    def is_function_keyword_parameter(self):
+        return self.tools.word_finder.is_function_keyword_parameter(self.offset)
+
     @property
     @utils.saveit
     def lineno(self):
@@ -138,13 +194,14 @@ def same_pyname(expected, pyname):
         return False
     if expected == pyname:
         return True
-    if type(expected) not in (pynames.ImportedModule, pynames.ImportedName) \
-        and type(pyname) not in \
-            (pynames.ImportedModule, pynames.ImportedName):
+    if type(expected) not in (pynames.ImportedModule, pynames.ImportedName) and type(
+        pyname
+    ) not in (pynames.ImportedModule, pynames.ImportedName):
         return False
-    return expected.get_definition_location() == \
-        pyname.get_definition_location() and \
-        expected.get_object() == pyname.get_object()
+    return (
+        expected.get_definition_location() == pyname.get_definition_location()
+        and expected.get_object() == pyname.get_object()
+    )
 
 
 def unsure_pyname(pyname, unbound=True):
@@ -157,8 +214,8 @@ def unsure_pyname(pyname, unbound=True):
         return True
 
 
-class PyNameFilter(object):
-    """For finding occurrences of a name"""
+class PyNameFilter:
+    """For finding occurrences of a name."""
 
     def __init__(self, pyname):
         self.pyname = pyname
@@ -168,8 +225,8 @@ class PyNameFilter(object):
             return True
 
 
-class InHierarchyFilter(object):
-    """For finding occurrences of a name"""
+class InHierarchyFilter:
+    """Finds the occurrence if the name is in the class's hierarchy."""
 
     def __init__(self, pyname, implementations_only=False):
         self.pyname = pyname
@@ -194,22 +251,23 @@ class InHierarchyFilter(object):
         if isinstance(pyname, pynames.DefinedName):
             scope = pyname.get_object().get_scope()
             parent = scope.parent
-            if parent is not None and parent.get_kind() == 'Class':
+            if parent is not None and parent.get_kind() == "Class":
                 return parent.pyobject
 
     def _get_root_classes(self, pyclass, name):
         if self.impl_only and pyclass == self.pyclass:
-            return set([pyclass])
+            return {pyclass}
         result = set()
         for superclass in pyclass.get_superclasses():
             if name in superclass:
                 result.update(self._get_root_classes(superclass, name))
         if not result:
-            return set([pyclass])
+            return {pyclass}
         return result
 
 
-class UnsureFilter(object):
+class UnsureFilter:
+    """Occurrences where we don't knoow what the name references."""
 
     def __init__(self, unsure):
         self.unsure = unsure
@@ -219,28 +277,41 @@ class UnsureFilter(object):
             return True
 
 
-class NoImportsFilter(object):
+class NoImportsFilter:
+    """Don't include import statements as occurrences."""
 
     def __call__(self, occurrence):
         if occurrence.is_in_import_statement():
             return False
 
 
-class CallsFilter(object):
+class CallsFilter:
+    """Filter out non-call occurrences."""
 
     def __call__(self, occurrence):
         if not occurrence.is_called():
             return False
 
 
-class _TextualFinder(object):
+class NoKeywordsFilter:
+    """Filter out keyword parameters."""
 
+    def __call__(self, occurrence):
+        if occurrence.is_function_keyword_parameter():
+            return False
+
+
+class _TextualFinder:
     def __init__(self, name, docs=False):
         self.name = name
         self.docs = docs
-        self.comment_pattern = _TextualFinder.any('comment', [r'#[^\n]*'])
+        self.comment_pattern = _TextualFinder.any("comment", [r"#[^\n]*"])
         self.string_pattern = _TextualFinder.any(
-            'string', [codeanalyze.get_string_pattern()])
+            "string", [codeanalyze.get_string_pattern()]
+        )
+        self.f_string_pattern = _TextualFinder.any(
+            "fstring", [codeanalyze.get_formatted_string_pattern()]
+        )
         self.pattern = self._get_occurrence_pattern(self.name)
 
     def find_offsets(self, source):
@@ -250,14 +321,22 @@ class _TextualFinder(object):
             searcher = self._normal_search
         else:
             searcher = self._re_search
-        for matched in searcher(source):
-            yield matched
+        yield from searcher(source)
 
     def _re_search(self, source):
         for match in self.pattern.finditer(source):
-            for key, value in match.groupdict().items():
-                if value and key == 'occurrence':
-                    yield match.start(key)
+            if match.groupdict()["occurrence"]:
+                yield match.start("occurrence")
+            elif utils.pycompat.PY36 and match.groupdict()["fstring"]:
+                f_string = match.groupdict()["fstring"]
+                for occurrence_node in self._search_in_f_string(f_string):
+                    yield match.start("fstring") + occurrence_node.col_offset
+
+    def _search_in_f_string(self, f_string):
+        tree = ast.parse(f_string)
+        for node in ast.walk(tree):
+            if isinstance(node, ast.Name) and node.id == self.name:
+                yield node
 
     def _normal_search(self, source):
         current = 0
@@ -265,16 +344,15 @@ class _TextualFinder(object):
             try:
                 found = source.index(self.name, current)
                 current = found + len(self.name)
-                if (found == 0 or
-                        not self._is_id_char(source[found - 1])) and \
-                    (current == len(source) or
-                        not self._is_id_char(source[current])):
+                if (found == 0 or not self._is_id_char(source[found - 1])) and (
+                    current == len(source) or not self._is_id_char(source[current])
+                ):
                     yield found
             except ValueError:
                 break
 
     def _is_id_char(self, c):
-        return c.isalnum() or c == '_'
+        return c.isalnum() or c == "_"
 
     def _fast_file_query(self, source):
         try:
@@ -290,21 +368,26 @@ class _TextualFinder(object):
             return pymodule.source_code
 
     def _get_occurrence_pattern(self, name):
-        occurrence_pattern = _TextualFinder.any('occurrence',
-                                                ['\\b' + name + '\\b'])
-        pattern = re.compile(occurrence_pattern + '|' + self.comment_pattern +
-                             '|' + self.string_pattern)
+        occurrence_pattern = _TextualFinder.any("occurrence", ["\\b" + name + "\\b"])
+        pattern = re.compile(
+            occurrence_pattern
+            + "|"
+            + self.comment_pattern
+            + "|"
+            + self.string_pattern
+            + "|"
+            + self.f_string_pattern
+        )
         return pattern
 
     @staticmethod
     def any(name, list_):
-        return '(?P<%s>' % name + '|'.join(list_) + ')'
+        return "(?P<%s>" % name + "|".join(list_) + ")"
 
 
-class _OccurrenceToolsCreator(object):
-
-    def __init__(self, pycore, resource=None, pymodule=None, docs=False):
-        self.pycore = pycore
+class _OccurrenceToolsCreator:
+    def __init__(self, project, resource=None, pymodule=None, docs=False):
+        self.project = project
         self.__resource = resource
         self.__pymodule = pymodule
         self.docs = docs
@@ -317,10 +400,7 @@ class _OccurrenceToolsCreator(object):
     @property
     @utils.saveit
     def source_code(self):
-        if self.__resource is not None:
-            return self.resource.read()
-        else:
-            return self.pymodule.source_code
+        return self.pymodule.source_code
 
     @property
     @utils.saveit
@@ -340,4 +420,4 @@ class _OccurrenceToolsCreator(object):
     def pymodule(self):
         if self.__pymodule is not None:
             return self.__pymodule
-        return self.pycore.resource_to_pyobject(self.resource)
+        return self.project.get_pymodule(self.resource)
