@@ -2,6 +2,7 @@ import collections
 import numbers
 import re
 import warnings
+from itertools import chain
 
 from rope.base import ast, codeanalyze, exceptions
 from rope.base.utils import pycompat
@@ -68,7 +69,7 @@ class MismatchedTokenError(exceptions.RopeError):
     pass
 
 
-class _PatchingASTWalker(object):
+class _PatchingASTWalker:
     def __init__(self, source, children=False):
         self.source = _Source(source)
         self.children = children
@@ -249,6 +250,24 @@ class _PatchingASTWalker(object):
                     return child.col_offset + self.lines.get_line_start(child.lineno)
         return len(self.source.source)
 
+    def _join(self, iterable, separator):
+        iterable = iter(iterable)
+        try:
+            yield next(iterable)
+        except StopIteration:
+            return
+        for child in iterable:
+            yield separator
+            yield child
+
+    def _flatten_keywords(self, iterable):
+        iterable = ([attr, "=", pattern] for attr, pattern in iterable)
+        iterable = self._join(iterable, separator=[","])
+        return chain.from_iterable(iterable)
+
+    def _child_nodes(self, nodes, separator):
+        return list(self._join(nodes, separator=separator))
+
     _operators = {
         "And": "and",
         "Or": "or",
@@ -900,19 +919,44 @@ class _PatchingASTWalker(object):
     def _AsyncWith(self, node):
         self._handle_with_node(node, is_async=True)
 
-    def _child_nodes(self, nodes, separator):
-        children = []
-        for index, child in enumerate(nodes):
-            children.append(child)
-            if index < len(nodes) - 1:
-                children.append(separator)
-        return children
-
     def _Starred(self, node):
         self._handle(node, [node.value])
 
+    def _Match(self, node):
+        children = ["match", node.subject, ":"]
+        children.extend(node.cases)
+        self._handle(node, children)
 
-class _Source(object):
+    def _match_case(self, node):
+        children = ["case", node.pattern]
+        if node.guard:
+            children.extend(["if", node.guard])
+        children.append(":")
+        children.extend(node.body)
+        self._handle(node, children)
+
+    def _MatchAs(self, node):
+        if node.pattern:
+            children = [node.pattern, "as", node.name]
+        elif node.name is None:
+            children = ["_"]
+        else:
+            children = [node.name]
+        self._handle(node, children)
+
+    def _MatchClass(self, node):
+        children = []
+        children.extend([node.cls, "("])
+        children.extend(self._child_nodes(node.patterns, ","))
+        children.extend(self._flatten_keywords(zip(node.kwd_attrs, node.kwd_patterns)))
+        children.append(")")
+        self._handle(node, children)
+
+    def _MatchValue(self, node):
+        self._handle(node, [node.value])
+
+
+class _Source:
     def __init__(self, source):
         self.source = source
         self.offset = 0
@@ -927,7 +971,7 @@ class _Source(object):
                     self._skip_comment()
         except (ValueError, TypeError) as e:
             raise MismatchedTokenError(
-                "Token <%s> at %s cannot be matched" % (token, self._get_location())
+                "Token <{}> at {} cannot be matched".format(token, self._get_location())
             )
         self.offset = new_offset + len(token)
         return (new_offset, self.offset)
@@ -941,8 +985,8 @@ class _Source(object):
         if _Source._string_pattern is None:
             string_pattern = codeanalyze.get_string_pattern()
             formatted_string_pattern = codeanalyze.get_formatted_string_pattern()
-            original = r"(?:%s)|(?:%s)" % (string_pattern, formatted_string_pattern)
-            pattern = r"(%s)((\s|\\\n|#[^\n]*\n)*(%s))*" % (original, original)
+            original = r"(?:{})|(?:{})".format(string_pattern, formatted_string_pattern)
+            pattern = r"({})((\s|\\\n|#[^\n]*\n)*({}))*".format(original, original)
             _Source._string_pattern = re.compile(pattern)
         repattern = _Source._string_pattern
         return self._consume_pattern(repattern, end)
